@@ -2,26 +2,29 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 1, 1)
+__version__ = (0, 1, 2)
 __all__ = [
     "thread_batch", "thread_pool_batch", "async_batch", 
     "threaded", "run_as_thread", "asynchronized", "run_as_async", 
-    "threadpool_map", "taskgroup_map", "conmap", "Return", 
+    "threadpool_map", "taskgroup_map", "conmap", "iter_pages", 
+    "Return", 
 ]
 
 from asyncio import (
-    ensure_future, get_event_loop, BaseEventLoop, CancelledError, Future as AsyncFuture, 
-    Queue as AsyncQueue, Semaphore as AsyncSemaphore, TaskGroup, 
+    ensure_future, get_event_loop, BaseEventLoop, CancelledError as AsyncCancelledError, 
+    Future as AsyncFuture, Queue as AsyncQueue, Semaphore as AsyncSemaphore, TaskGroup, 
 )
 from collections.abc import (
     AsyncIterable, AsyncIterator, Awaitable, Callable, Coroutine, Iterable, 
     Iterator, Mapping, 
 )
-from concurrent.futures import Executor, Future, ThreadPoolExecutor
+from concurrent.futures import CancelledError, Executor, Future, ThreadPoolExecutor
 from functools import partial, update_wrapper
-from inspect import isawaitable, iscoroutinefunction, signature
+from inspect import isawaitable, iscoroutinefunction, signature, Signature
+from itertools import count
 from os import cpu_count
 from queue import Queue, SimpleQueue, Empty
+from sys import exc_info
 from _thread import start_new_thread
 from threading import Event, Lock, Semaphore, Thread
 from typing import cast, overload, Any, ContextManager, Literal
@@ -29,6 +32,7 @@ from typing import cast, overload, Any, ContextManager, Literal
 from argtools import argcount
 from asynctools import async_map, async_zip, ensure_coroutine, run_async
 from decotools import optional
+from iterutils import run_gen_step, run_gen_step_iter, Yield
 
 
 if "__del__" not in ThreadPoolExecutor.__dict__:
@@ -42,6 +46,17 @@ class Return:
         self.value = value
 
 
+def has_keyword_async(request: Callable | Signature, /) -> bool:
+    if callable(request):
+        try:
+            request = signature(request)
+        except (ValueError, TypeError):
+            return False
+    params = request.parameters
+    param = params.get("async_")
+    return bool(param and param.kind in (param.POSITIONAL_OR_KEYWORD, param.KEYWORD_ONLY))
+
+
 def thread_batch[T, V](
     work: Callable[[T], V] | Callable[[T, Callable], V], 
     tasks: Iterable[T], 
@@ -188,7 +203,7 @@ async def async_batch[T, V](
         except KeyboardInterrupt:
             raise
         except BaseException as e:
-            raise CancelledError from e
+            raise AsyncCancelledError from e
     async with TaskGroup() as tg:
         create_task = tg.create_task
         submit = lambda task, /: create_task(works(task))
@@ -422,12 +437,9 @@ def conmap[T](
     kwargs: Mapping = {}, 
     async_: Literal[False, True] = False, 
 ) -> Iterator[T] | AsyncIterator[T]:
-    map: Callable = taskgroup_map if async_ else threadpool_map
-    params = signature(func).parameters
-    if ((param := params.get("async_")) and 
-        (param.kind in (param.POSITIONAL_OR_KEYWORD, param.KEYWORD_ONLY))
-    ):
+    if has_keyword_async(func):
         kwargs = {"async_": async_, **kwargs}
+    map: Callable = taskgroup_map if async_ else threadpool_map
     return map(
         func, 
         it, 
@@ -437,3 +449,178 @@ def conmap[T](
         kwargs=kwargs, 
     )
 
+
+@overload
+def iter_pages[T](
+    func: Callable[[int], T], 
+    is_last_page: Callable[[T], bool], 
+    next_page: int | Callable[[], int] | Iterable[int] = 1, 
+    max_workers: None | int = 1, 
+    unordered: bool = True, 
+    *, 
+    async_: Literal[False] = False, 
+) -> Iterator[T]:
+    ...
+@overload
+def iter_pages[T](
+    func: Callable[[int], T], 
+    is_last_page: Callable[[T], bool], 
+    next_page: int | Callable[[], int] | Iterable[int] = 1, 
+    max_workers: None | int = 1, 
+    unordered: bool = True, 
+    *, 
+    async_: Literal[True], 
+) -> AsyncIterator[T]:
+    ...
+def iter_pages[T](
+    func: Callable[[int], T], 
+    is_last_page: Callable[[T], bool], 
+    next_page: int | Callable[[], int] | Iterable[int] = 1, 
+    max_workers: None | int = 1, 
+    unordered: bool = True, 
+    *, 
+    async_: Literal[False, True] = False, 
+) -> Iterator[T] | AsyncIterator[T]:
+    """分页执行操作
+
+    :param func: 操作函数
+    :param is_last_page: 根据响应判断是否下一页，执行后可以报错
+    :param next_page: 获取下一页
+    :param max_workers: 最大并发数，如果为 None 或 <= 0，则自动确定
+    :param unordered: 是否允许乱序
+    :param async_: 是否异步
+
+    :return: 迭代器，产生每次操作的响应
+    """
+    if iscoroutinefunction(func):
+        async_ = True
+    if has_keyword_async(func):
+        func = partial(func, async_=async_) # type: ignore
+    if async_:
+        if max_workers is None or max_workers <= 0:
+            max_workers = 20
+    elif max_workers is not None and max_workers <= 0:
+        max_workers = None
+    if isinstance(next_page, int):
+        next_page = count(next_page).__next__
+    elif not callable(next_page):
+        next_page = iter(next_page).__next__
+    if max_workers == 1:
+        def gen_step():
+            try:
+                while True:
+                    page = next_page()
+                    resp = yield func(page)
+                    if is_last_page(resp):
+                        yield Yield(resp)
+                        break
+                    yield Yield(resp)
+            except StopIteration:
+                pass
+    else:
+        def gen_step():
+            if async_:
+                q: Any = AsyncQueue()
+            else:
+                q = SimpleQueue()
+            get, put = q.get, q.put_nowait
+            if async_:
+                n = cast(int, max_workers)
+                task_group = TaskGroup()
+                yield task_group.__aenter__()
+                create_task = task_group.create_task
+                submit: Callable = lambda f, /, *a, **k: create_task(f(*a, **k))
+                shutdown: Callable = lambda: task_group.__aexit__(*exc_info())
+            else:
+                executor = ThreadPoolExecutor(max_workers)
+                n = executor._max_workers
+                submit = executor.submit
+                shutdown = lambda: executor.shutdown(False, cancel_futures=True)
+            sentinel = object()
+            countdown: Callable
+            if async_:
+                def countdown(_, /):
+                    nonlocal n
+                    n -= 1
+                    if not n:
+                        put(sentinel)
+            else:
+                def countdown(_, /, lock=Lock()):
+                    nonlocal n
+                    with lock:
+                        n -= 1
+                        if not n:
+                            put(sentinel)
+            task_list: list = [None] * n
+            task_page: list[int] = [0] * n
+            max_page: None | int = None
+            def request(task_idx, /):
+                nonlocal max_page
+                page = 0
+                try:
+                    if unordered:
+                        while True:
+                            try:
+                                page = next_page()
+                                if max_page is not None and page > max_page:
+                                    return
+                                task_page[task_idx] = page
+                                resp = yield func(page)
+                                if is_last_page(resp):
+                                    put(resp)
+                                    break
+                                put(resp)
+                            except StopIteration:
+                                break
+                            except BaseException as e:
+                                put(e)
+                                return
+                    else:
+                        while True:
+                            page = next_page()
+                            if max_page is not None and page > max_page:
+                                return
+                            task_page[task_idx] = page
+                            resp = yield func(page)
+                            if is_last_page(resp):
+                                put(resp)
+                                break
+                            put(resp)
+                finally:
+                    if max_page is not None:
+                        max_page = page
+                        for i, p in enumerate(task_page):
+                            if p > page:
+                                task_list[i].cancel()
+            try:
+                for i in range(n):
+                    task = task_list[i] = submit(run_gen_step, request(i), async_)
+                    task.add_done_callback(countdown)
+                if unordered:
+                    while True:
+                        resp = yield get()
+                        if resp is sentinel:
+                            break
+                        elif isinstance(resp, (CancelledError, AsyncCancelledError)):
+                            continue
+                        elif isinstance(resp, BaseException):
+                            raise resp
+                        yield Yield(resp)
+                else:
+                    while True:
+                        task = yield get()
+                        try:
+                            if async_:
+                                resp = yield task
+                            else:
+                                resp = task.result()
+                        except (StopIteration, CancelledError, AsyncCancelledError):
+                            continue
+                        if resp is sentinel:
+                            break
+                        yield Yield(resp)
+            finally:
+                yield shutdown()
+    return run_gen_step_iter(gen_step, async_)
+
+# TODO: iter_pages_with_cooldown
